package persistance.services;

import java.util.List;
import java.util.Optional;

import javax.persistence.EntityManager;
import javax.persistence.EntityTransaction;
import javax.persistence.TypedQuery;
import javax.ws.rs.core.Response;

import org.jvnet.hk2.annotations.Service;

import jakarta.inject.Inject;
import jakarta.inject.Provider;
import persistance.interfaces.Iabm;

@Service
public class ABMService<T> implements Iabm<T>{
    private Class<T> entityClass;
    
    @Inject
    private Provider<EntityManager> emProvider;
    
    public ABMService(Class<T> entityClass) {
        this.entityClass = entityClass;
    }
    
	@Override
	public Optional<T> get(long id) {
		EntityManager em = emProvider.get();
		T entidad = null;
		try {
	        entidad = em.find(entityClass, id);
		}catch (Exception e) {
		    e.printStackTrace();
		}finally {
		    if (em != null) {
		        em.close();
		    }
		}
		return Optional.ofNullable(entidad);
	}

	@Override
	public List<T> getAll() {
		EntityManager em = emProvider.get();
		List<T> entidades = null;
		try {
	        TypedQuery<T> query = em.createQuery("SELECT e FROM " + entityClass.getSimpleName() + " e", entityClass);
	        entidades = query.getResultList();
		}catch (Exception e) {
		    e.printStackTrace();
		}finally {
		    if (em != null) {
		        em.close();
		    }
		}
		return entidades;
	}

    @Override
    public Response.Status save(T entidad) {
        return transaccion(entidad, EntityOperation.PERSIST);
    }

    @Override
    public Response.Status update(T entidad) {
        return transaccion(entidad, EntityOperation.MERGE);
    }

    @Override
    public Response.Status delete(T entidad) {
        return transaccion(entidad, EntityOperation.REMOVE);
    }

    private Response.Status transaccion(T entidad, EntityOperation operation) {
		EntityManager em = emProvider.get();
        Response.Status status = Response.Status.INTERNAL_SERVER_ERROR;
        try {
            EntityTransaction etx = em.getTransaction();
            etx.begin();
            switch (operation) {
                case PERSIST:
                    em.persist(entidad);
                    status = Response.Status.CREATED;
                    break;
                case MERGE:
                    em.merge(entidad);
                    status = Response.Status.OK;
                    break;
                case REMOVE:
                    em.remove(em.contains(entidad) ? entidad : em.merge(entidad));
                    status = Response.Status.NO_CONTENT;
                    break;
            }
            etx.commit();
        } catch (Exception e) {
            if (em != null && em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            e.printStackTrace();
            status = Response.Status.CONFLICT;
        } finally {
            if (em != null) {
                em.close();
            }
        }
        return status;
    }
    
    private enum EntityOperation {
        PERSIST,
        MERGE,
        REMOVE
    }
}